/****************************************************************************
** Meta object code from reading C++ file 'qgeotileproviderosm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../qtlocation/src/plugins/geoservices/osm/qgeotileproviderosm.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeotileproviderosm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12TileProviderE_t {};
} // unnamed namespace

template <> constexpr inline auto TileProvider::qt_create_metaobjectdata<qt_meta_tag_ZN12TileProviderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TileProvider",
        "resolutionFinished",
        "",
        "TileProvider*",
        "provider",
        "resolutionError",
        "onNetworkReplyFinished",
        "onNetworkReplyError",
        "QNetworkReply::NetworkError",
        "error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resolutionFinished'
        QtMocHelpers::SignalData<void(TileProvider *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'resolutionError'
        QtMocHelpers::SignalData<void(TileProvider *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onNetworkReplyFinished'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onNetworkReplyError'
        QtMocHelpers::SlotData<void(QNetworkReply::NetworkError)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TileProvider, qt_meta_tag_ZN12TileProviderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TileProvider::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TileProviderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TileProviderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12TileProviderE_t>.metaTypes,
    nullptr
} };

void TileProvider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TileProvider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resolutionFinished((*reinterpret_cast<std::add_pointer_t<TileProvider*>>(_a[1]))); break;
        case 1: _t->resolutionError((*reinterpret_cast<std::add_pointer_t<TileProvider*>>(_a[1]))); break;
        case 2: _t->onNetworkReplyFinished(); break;
        case 3: _t->onNetworkReplyError((*reinterpret_cast<std::add_pointer_t<QNetworkReply::NetworkError>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TileProvider* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TileProvider* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QNetworkReply::NetworkError >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TileProvider::*)(TileProvider * )>(_a, &TileProvider::resolutionFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TileProvider::*)(TileProvider * )>(_a, &TileProvider::resolutionError, 1))
            return;
    }
}

const QMetaObject *TileProvider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TileProvider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TileProviderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TileProvider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void TileProvider::resolutionFinished(TileProvider * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TileProvider::resolutionError(TileProvider * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN19QGeoTileProviderOsmE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoTileProviderOsm::qt_create_metaobjectdata<qt_meta_tag_ZN19QGeoTileProviderOsmE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoTileProviderOsm",
        "resolutionFinished",
        "",
        "const QGeoTileProviderOsm*",
        "provider",
        "resolutionError",
        "resolutionRequired",
        "resolveProvider",
        "disableRedirection",
        "onResolutionFinished",
        "TileProvider*",
        "onResolutionError",
        "updateCameraCapabilities"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'resolutionFinished'
        QtMocHelpers::SignalData<void(const QGeoTileProviderOsm *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'resolutionError'
        QtMocHelpers::SignalData<void(const QGeoTileProviderOsm *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'resolutionRequired'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resolveProvider'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'disableRedirection'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onResolutionFinished'
        QtMocHelpers::SlotData<void(TileProvider *)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 4 },
        }}),
        // Slot 'onResolutionError'
        QtMocHelpers::SlotData<void(TileProvider *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 4 },
        }}),
        // Slot 'updateCameraCapabilities'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QGeoTileProviderOsm, qt_meta_tag_ZN19QGeoTileProviderOsmE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoTileProviderOsm::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoTileProviderOsmE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoTileProviderOsmE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QGeoTileProviderOsmE_t>.metaTypes,
    nullptr
} };

void QGeoTileProviderOsm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGeoTileProviderOsm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->resolutionFinished((*reinterpret_cast<std::add_pointer_t<const QGeoTileProviderOsm*>>(_a[1]))); break;
        case 1: _t->resolutionError((*reinterpret_cast<std::add_pointer_t<const QGeoTileProviderOsm*>>(_a[1]))); break;
        case 2: _t->resolutionRequired(); break;
        case 3: _t->resolveProvider(); break;
        case 4: _t->disableRedirection(); break;
        case 5: _t->onResolutionFinished((*reinterpret_cast<std::add_pointer_t<TileProvider*>>(_a[1]))); break;
        case 6: _t->onResolutionError((*reinterpret_cast<std::add_pointer_t<TileProvider*>>(_a[1]))); break;
        case 7: _t->updateCameraCapabilities(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TileProvider* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TileProvider* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QGeoTileProviderOsm::*)(const QGeoTileProviderOsm * )>(_a, &QGeoTileProviderOsm::resolutionFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoTileProviderOsm::*)(const QGeoTileProviderOsm * )>(_a, &QGeoTileProviderOsm::resolutionError, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoTileProviderOsm::*)()>(_a, &QGeoTileProviderOsm::resolutionRequired, 2))
            return;
    }
}

const QMetaObject *QGeoTileProviderOsm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGeoTileProviderOsm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QGeoTileProviderOsmE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QGeoTileProviderOsm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QGeoTileProviderOsm::resolutionFinished(const QGeoTileProviderOsm * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QGeoTileProviderOsm::resolutionError(const QGeoTileProviderOsm * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QGeoTileProviderOsm::resolutionRequired()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
