/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeomapquickitem_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/quickmapitems/qdeclarativegeomapquickitem_p.h"
#include <QtNetwork/QSslError>
#include <QtLocation/private/qdeclarativegeoserviceprovider_p.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeomapquickitem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27QDeclarativeGeoMapQuickItemE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoMapQuickItem::qt_create_metaobjectdata<qt_meta_tag_ZN27QDeclarativeGeoMapQuickItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoMapQuickItem",
        "QML.Element",
        "MapQuickItem",
        "QML.AddedInVersion",
        "1280",
        "coordinateChanged",
        "",
        "sourceItemChanged",
        "anchorPointChanged",
        "zoomLevelChanged",
        "afterChildrenChanged",
        "afterViewportChanged",
        "QGeoMapViewportChangeEvent",
        "event",
        "coordinate",
        "QGeoCoordinate",
        "anchorPoint",
        "QPointF",
        "zoomLevel",
        "sourceItem",
        "QQuickItem*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'coordinateChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceItemChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'anchorPointChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zoomLevelChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'afterChildrenChanged'
        QtMocHelpers::SlotData<void()>(10, 6, QMC::AccessProtected, QMetaType::Void),
        // Slot 'afterViewportChanged'
        QtMocHelpers::SlotData<void(const QGeoMapViewportChangeEvent &)>(11, 6, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'coordinate'
        QtMocHelpers::PropertyData<QGeoCoordinate>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'anchorPoint'
        QtMocHelpers::PropertyData<QPointF>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'zoomLevel'
        QtMocHelpers::PropertyData<qreal>(18, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'sourceItem'
        QtMocHelpers::PropertyData<QQuickItem*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoMapQuickItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeoMapQuickItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QDeclarativeGeoMapItemBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeGeoMapQuickItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeGeoMapQuickItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27QDeclarativeGeoMapQuickItemE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoMapQuickItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoMapQuickItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->coordinateChanged(); break;
        case 1: _t->sourceItemChanged(); break;
        case 2: _t->anchorPointChanged(); break;
        case 3: _t->zoomLevelChanged(); break;
        case 4: _t->afterChildrenChanged(); break;
        case 5: _t->afterViewportChanged((*reinterpret_cast<std::add_pointer_t<QGeoMapViewportChangeEvent>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapQuickItem::*)()>(_a, &QDeclarativeGeoMapQuickItem::coordinateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapQuickItem::*)()>(_a, &QDeclarativeGeoMapQuickItem::sourceItemChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapQuickItem::*)()>(_a, &QDeclarativeGeoMapQuickItem::anchorPointChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapQuickItem::*)()>(_a, &QDeclarativeGeoMapQuickItem::zoomLevelChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->coordinate(); break;
        case 1: *reinterpret_cast<QPointF*>(_v) = _t->anchorPoint(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->zoomLevel(); break;
        case 3: *reinterpret_cast<QQuickItem**>(_v) = _t->sourceItem(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCoordinate(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 1: _t->setAnchorPoint(*reinterpret_cast<QPointF*>(_v)); break;
        case 2: _t->setZoomLevel(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setSourceItem(*reinterpret_cast<QQuickItem**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoMapQuickItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoMapQuickItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27QDeclarativeGeoMapQuickItemE_t>.strings))
        return static_cast<void*>(this);
    return QDeclarativeGeoMapItemBase::qt_metacast(_clname);
}

int QDeclarativeGeoMapQuickItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDeclarativeGeoMapItemBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoMapQuickItem::coordinateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeGeoMapQuickItem::sourceItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoMapQuickItem::anchorPointChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeoMapQuickItem::zoomLevelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
