/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativerectanglemapitem_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/quickmapitems/qdeclarativerectanglemapitem_p.h"
#include <QtNetwork/QSslError>
#include <QtLocation/private/qdeclarativegeoserviceprovider_p.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativerectanglemapitem_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28QDeclarativeRectangleMapItemE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeRectangleMapItem::qt_create_metaobjectdata<qt_meta_tag_ZN28QDeclarativeRectangleMapItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeRectangleMapItem",
        "QML.Element",
        "MapRectangle",
        "QML.AddedInVersion",
        "1280",
        "topLeftChanged",
        "",
        "QGeoCoordinate",
        "topLeft",
        "bottomRightChanged",
        "bottomRight",
        "colorChanged",
        "QColor",
        "color",
        "markSourceDirtyAndUpdate",
        "onLinePropertiesChanged",
        "afterViewportChanged",
        "QGeoMapViewportChangeEvent",
        "event",
        "border",
        "QDeclarativeMapLineProperties*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'topLeftChanged'
        QtMocHelpers::SignalData<void(const QGeoCoordinate &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'bottomRightChanged'
        QtMocHelpers::SignalData<void(const QGeoCoordinate &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 10 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'markSourceDirtyAndUpdate'
        QtMocHelpers::SlotData<void()>(14, 6, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onLinePropertiesChanged'
        QtMocHelpers::SlotData<void()>(15, 6, QMC::AccessProtected, QMetaType::Void),
        // Slot 'afterViewportChanged'
        QtMocHelpers::SlotData<void(const QGeoMapViewportChangeEvent &)>(16, 6, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'topLeft'
        QtMocHelpers::PropertyData<QGeoCoordinate>(8, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'bottomRight'
        QtMocHelpers::PropertyData<QGeoCoordinate>(10, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'border'
        QtMocHelpers::PropertyData<QDeclarativeMapLineProperties*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeRectangleMapItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeRectangleMapItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QDeclarativeGeoMapItemBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDeclarativeRectangleMapItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDeclarativeRectangleMapItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QDeclarativeRectangleMapItemE_t>.metaTypes,
    nullptr
} };

void QDeclarativeRectangleMapItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeRectangleMapItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->topLeftChanged((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 1: _t->bottomRightChanged((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 2: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->markSourceDirtyAndUpdate(); break;
        case 4: _t->onLinePropertiesChanged(); break;
        case 5: _t->afterViewportChanged((*reinterpret_cast<std::add_pointer_t<QGeoMapViewportChangeEvent>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeRectangleMapItem::*)(const QGeoCoordinate & )>(_a, &QDeclarativeRectangleMapItem::topLeftChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeRectangleMapItem::*)(const QGeoCoordinate & )>(_a, &QDeclarativeRectangleMapItem::bottomRightChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeRectangleMapItem::*)(const QColor & )>(_a, &QDeclarativeRectangleMapItem::colorChanged, 2))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeMapLineProperties* >(); break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->topLeft(); break;
        case 1: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->bottomRight(); break;
        case 2: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 3: *reinterpret_cast<QDeclarativeMapLineProperties**>(_v) = _t->border(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTopLeft(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 1: _t->setBottomRight(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 2: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeRectangleMapItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeRectangleMapItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QDeclarativeRectangleMapItemE_t>.strings))
        return static_cast<void*>(this);
    return QDeclarativeGeoMapItemBase::qt_metacast(_clname);
}

int QDeclarativeRectangleMapItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDeclarativeGeoMapItemBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeRectangleMapItem::topLeftChanged(const QGeoCoordinate & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDeclarativeRectangleMapItem::bottomRightChanged(const QGeoCoordinate & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QDeclarativeRectangleMapItem::colorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
