/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeomapcopyrightsnotice_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/quickmapitems/qdeclarativegeomapcopyrightsnotice_p.h"
#include <QtLocation/private/qdeclarativegeomap_p.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeomapcopyrightsnotice_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN33QDeclarativeGeoMapCopyrightNoticeE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoMapCopyrightNotice::qt_create_metaobjectdata<qt_meta_tag_ZN33QDeclarativeGeoMapCopyrightNoticeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoMapCopyrightNotice",
        "QML.Element",
        "MapCopyrightNotice",
        "QML.AddedInVersion",
        "1280",
        "linkActivated",
        "",
        "link",
        "mapSourceChanged",
        "backgroundColorChanged",
        "QColor",
        "color",
        "styleSheetChanged",
        "styleSheet",
        "copyrightsVisibleChanged",
        "copyrightsImageChanged",
        "QImage",
        "copyrightsImage",
        "copyrightsChanged",
        "copyrightsHtml",
        "onCopyrightsStyleSheetChanged",
        "mapSource",
        "QDeclarativeGeoMap*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'linkActivated'
        QtMocHelpers::SignalData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'mapSourceChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backgroundColorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'styleSheetChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'copyrightsVisibleChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyrightsImageChanged'
        QtMocHelpers::SlotData<void(const QImage &)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'copyrightsChanged'
        QtMocHelpers::SlotData<void(const QString &)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'onCopyrightsStyleSheetChanged'
        QtMocHelpers::SlotData<void(const QString &)>(20, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'mapSource'
        QtMocHelpers::PropertyData<QDeclarativeGeoMap*>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'styleSheet'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoMapCopyrightNotice, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeoMapCopyrightNotice::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickPaintedItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33QDeclarativeGeoMapCopyrightNoticeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33QDeclarativeGeoMapCopyrightNoticeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33QDeclarativeGeoMapCopyrightNoticeE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoMapCopyrightNotice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoMapCopyrightNotice *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->linkActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->mapSourceChanged(); break;
        case 2: _t->backgroundColorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 3: _t->styleSheetChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->copyrightsVisibleChanged(); break;
        case 5: _t->copyrightsImageChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 6: _t->copyrightsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->onCopyrightsStyleSheetChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapCopyrightNotice::*)(const QString & )>(_a, &QDeclarativeGeoMapCopyrightNotice::linkActivated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapCopyrightNotice::*)()>(_a, &QDeclarativeGeoMapCopyrightNotice::mapSourceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapCopyrightNotice::*)(const QColor & )>(_a, &QDeclarativeGeoMapCopyrightNotice::backgroundColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapCopyrightNotice::*)(const QString & )>(_a, &QDeclarativeGeoMapCopyrightNotice::styleSheetChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapCopyrightNotice::*)()>(_a, &QDeclarativeGeoMapCopyrightNotice::copyrightsVisibleChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativeGeoMap**>(_v) = _t->mapSource(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->styleSheet(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMapSource(*reinterpret_cast<QDeclarativeGeoMap**>(_v)); break;
        case 1: _t->setStyleSheet(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoMapCopyrightNotice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoMapCopyrightNotice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33QDeclarativeGeoMapCopyrightNoticeE_t>.strings))
        return static_cast<void*>(this);
    return QQuickPaintedItem::qt_metacast(_clname);
}

int QDeclarativeGeoMapCopyrightNotice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickPaintedItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoMapCopyrightNotice::linkActivated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDeclarativeGeoMapCopyrightNotice::mapSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoMapCopyrightNotice::backgroundColorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QDeclarativeGeoMapCopyrightNotice::styleSheetChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QDeclarativeGeoMapCopyrightNotice::copyrightsVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
