/****************************************************************************
** Meta object code from reading C++ file 'qgeoroute.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/maps/qgeoroute.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeoroute.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9QGeoRouteE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoRoute::qt_create_metaobjectdata<qt_meta_tag_ZN9QGeoRouteE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoRoute",
        "QML.Element",
        "route",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "routeId",
        "bounds",
        "QGeoRectangle",
        "travelTime",
        "distance",
        "path",
        "QList<QGeoCoordinate>",
        "routeLegs",
        "QList<QGeoRoute>",
        "extendedAttributes",
        "QVariantMap",
        "legIndex",
        "overallRoute",
        "segmentsCount",
        "qsizetype",
        "segments",
        "QList<QGeoRouteSegment>"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'routeId'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'bounds'
        QtMocHelpers::PropertyData<QGeoRectangle>(8, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'travelTime'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'distance'
        QtMocHelpers::PropertyData<qreal>(11, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'path'
        QtMocHelpers::PropertyData<QList<QGeoCoordinate>>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'routeLegs'
        QtMocHelpers::PropertyData<QList<QGeoRoute>>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'extendedAttributes'
        QtMocHelpers::PropertyData<QVariantMap>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'legIndex'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'overallRoute'
        QtMocHelpers::PropertyData<QGeoRoute>(19, 0x80000000 | 0, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'segmentsCount'
        QtMocHelpers::PropertyData<qsizetype>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'segments'
        QtMocHelpers::PropertyData<QList<QGeoRouteSegment>>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QGeoRoute, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoRoute::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QGeoRouteE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9QGeoRouteE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9QGeoRouteE_t>.metaTypes,
    nullptr
} };

void QGeoRoute::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoRoute *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QGeoCoordinate> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->routeId(); break;
        case 1: *reinterpret_cast<QGeoRectangle*>(_v) = _t->bounds(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->travelTime(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->distance(); break;
        case 4: *reinterpret_cast<QList<QGeoCoordinate>*>(_v) = _t->path(); break;
        case 5: *reinterpret_cast<QList<QGeoRoute>*>(_v) = _t->routeLegs(); break;
        case 6: *reinterpret_cast<QVariantMap*>(_v) = _t->extendedAttributes(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->legIndex(); break;
        case 8: *reinterpret_cast<QGeoRoute*>(_v) = _t->overallRoute(); break;
        case 9: *reinterpret_cast<qsizetype*>(_v) = _t->segmentsCount(); break;
        case 10: *reinterpret_cast<QList<QGeoRouteSegment>*>(_v) = _t->segments(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->setPath(*reinterpret_cast<QList<QGeoCoordinate>*>(_v)); break;
        default: break;
        }
    }
}
QT_WARNING_POP
