/****************************************************************************
** Meta object code from reading C++ file 'qgeomaptype_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/maps/qgeomaptype_p.h"
#include <QtLocation/private/qgeocameracapabilities_p.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeomaptype_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11QGeoMapTypeE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoMapType::qt_create_metaobjectdata<qt_meta_tag_ZN11QGeoMapTypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoMapType",
        "QML.Element",
        "mapType",
        "QML.Creatable",
        "true",
        "QML.CreationMethod",
        "structured",
        "style",
        "MapStyle",
        "name",
        "description",
        "mobile",
        "night",
        "cameraCapabilities",
        "QGeoCameraCapabilities",
        "metadata",
        "QVariantMap",
        "NoMap",
        "StreetMap",
        "SatelliteMapDay",
        "SatelliteMapNight",
        "TerrainMap",
        "HybridMap",
        "TransitMap",
        "GrayStreetMap",
        "PedestrianMap",
        "CarNavigationMap",
        "CycleMap",
        "CustomMap"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'style'
        QtMocHelpers::PropertyData<enum MapStyle>(7, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'mobile'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'night'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'cameraCapabilities'
        QtMocHelpers::PropertyData<QGeoCameraCapabilities>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'metadata'
        QtMocHelpers::PropertyData<QVariantMap>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'MapStyle'
        QtMocHelpers::EnumData<enum MapStyle>(8, 8, QMC::EnumFlags{}).add({
            {   17, MapStyle::NoMap },
            {   18, MapStyle::StreetMap },
            {   19, MapStyle::SatelliteMapDay },
            {   20, MapStyle::SatelliteMapNight },
            {   21, MapStyle::TerrainMap },
            {   22, MapStyle::HybridMap },
            {   23, MapStyle::TransitMap },
            {   24, MapStyle::GrayStreetMap },
            {   25, MapStyle::PedestrianMap },
            {   26, MapStyle::CarNavigationMap },
            {   27, MapStyle::CycleMap },
            {   28, MapStyle::CustomMap },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QGeoMapType, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QGeoMapType::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGeoMapTypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11QGeoMapTypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11QGeoMapTypeE_t>.metaTypes,
    nullptr
} };

void QGeoMapType::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<QGeoMapType *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum MapStyle*>(_v) = _t->style(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->mobile(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->night(); break;
        case 5: *reinterpret_cast<QGeoCameraCapabilities*>(_v) = _t->cameraCapabilities(); break;
        case 6: *reinterpret_cast<QVariantMap*>(_v) = _t->metadata(); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN27QGeoMapTypeForeignNamespaceE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoMapTypeForeignNamespace::qt_create_metaobjectdata<qt_meta_tag_ZN27QGeoMapTypeForeignNamespaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoMapTypeForeignNamespace",
        "QML.Foreign",
        "QGeoMapType",
        "QML.ForeignIsNamespace",
        "true",
        "QML.Element",
        "MapType"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN27QGeoMapTypeForeignNamespaceE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN27QGeoMapTypeForeignNamespaceE =
    QGeoMapTypeForeignNamespace::qt_create_metaobjectdata<qt_meta_tag_ZN27QGeoMapTypeForeignNamespaceE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN27QGeoMapTypeForeignNamespaceE =
    qt_staticMetaObjectContent_ZN27QGeoMapTypeForeignNamespaceE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN27QGeoMapTypeForeignNamespaceE =
    qt_staticMetaObjectContent_ZN27QGeoMapTypeForeignNamespaceE.relocatingData;

Q_CONSTINIT const QMetaObject QGeoMapTypeForeignNamespace::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN27QGeoMapTypeForeignNamespaceE.stringdata,
    qt_staticMetaObjectStaticContent_ZN27QGeoMapTypeForeignNamespaceE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN27QGeoMapTypeForeignNamespaceE.metaTypes,
    nullptr
} };

QT_WARNING_POP
