/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativecategory_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativeplaces/qdeclarativecategory_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativecategory_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20QDeclarativeCategoryE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeCategory::qt_create_metaobjectdata<qt_meta_tag_ZN20QDeclarativeCategoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeCategory",
        "QML.Element",
        "Category",
        "QML.AddedInVersion",
        "1280",
        "pluginChanged",
        "",
        "categoryIdChanged",
        "nameChanged",
        "visibilityChanged",
        "iconChanged",
        "statusChanged",
        "replyFinished",
        "pluginReady",
        "errorString",
        "save",
        "parentId",
        "remove",
        "category",
        "QPlaceCategory",
        "plugin",
        "QDeclarativeGeoServiceProvider*",
        "categoryId",
        "name",
        "visibility",
        "Visibility",
        "icon",
        "QPlaceIcon",
        "status",
        "Status",
        "UnspecifiedVisibility",
        "DeviceVisibility",
        "PrivateVisibility",
        "PublicVisibility",
        "Ready",
        "Saving",
        "Removing",
        "Error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pluginChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'categoryIdChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'nameChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibilityChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'replyFinished'
        QtMocHelpers::SlotData<void()>(12, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pluginReady'
        QtMocHelpers::SlotData<void()>(13, 6, QMC::AccessPrivate, QMetaType::Void),
        // Method 'errorString'
        QtMocHelpers::MethodData<QString() const>(14, 6, QMC::AccessPublic, QMetaType::QString),
        // Method 'save'
        QtMocHelpers::MethodData<void(const QString &)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Method 'save'
        QtMocHelpers::MethodData<void()>(15, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'remove'
        QtMocHelpers::MethodData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'category'
        QtMocHelpers::PropertyData<QPlaceCategory>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'plugin'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider*>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'categoryId'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'visibility'
        QtMocHelpers::PropertyData<enum Visibility>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'icon'
        QtMocHelpers::PropertyData<QPlaceIcon>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Visibility'
        QtMocHelpers::EnumData<enum Visibility>(25, 25, QMC::EnumFlags{}).add({
            {   30, Visibility::UnspecifiedVisibility },
            {   31, Visibility::DeviceVisibility },
            {   32, Visibility::PrivateVisibility },
            {   33, Visibility::PublicVisibility },
        }),
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(29, 29, QMC::EnumFlags{}).add({
            {   34, Status::Ready },
            {   35, Status::Saving },
            {   36, Status::Removing },
            {   37, Status::Error },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeCategory, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeCategory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDeclarativeCategoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDeclarativeCategoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20QDeclarativeCategoryE_t>.metaTypes,
    nullptr
} };

void QDeclarativeCategory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeCategory *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pluginChanged(); break;
        case 1: _t->categoryIdChanged(); break;
        case 2: _t->nameChanged(); break;
        case 3: _t->visibilityChanged(); break;
        case 4: _t->iconChanged(); break;
        case 5: _t->statusChanged(); break;
        case 6: _t->replyFinished(); break;
        case 7: _t->pluginReady(); break;
        case 8: { QString _r = _t->errorString();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->save((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->save(); break;
        case 11: _t->remove(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeCategory::*)()>(_a, &QDeclarativeCategory::pluginChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeCategory::*)()>(_a, &QDeclarativeCategory::categoryIdChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeCategory::*)()>(_a, &QDeclarativeCategory::nameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeCategory::*)()>(_a, &QDeclarativeCategory::visibilityChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeCategory::*)()>(_a, &QDeclarativeCategory::iconChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeCategory::*)()>(_a, &QDeclarativeCategory::statusChanged, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoServiceProvider* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPlaceCategory >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QPlaceIcon >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QPlaceCategory*>(_v) = _t->category(); break;
        case 1: *reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v) = _t->plugin(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->categoryId(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 4: *reinterpret_cast<enum Visibility*>(_v) = _t->visibility(); break;
        case 5: *reinterpret_cast<QPlaceIcon*>(_v) = _t->icon(); break;
        case 6: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCategory(*reinterpret_cast<QPlaceCategory*>(_v)); break;
        case 1: _t->setPlugin(*reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v)); break;
        case 2: _t->setCategoryId(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setVisibility(*reinterpret_cast<enum Visibility*>(_v)); break;
        case 5: _t->setIcon(*reinterpret_cast<QPlaceIcon*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeCategory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeCategory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20QDeclarativeCategoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeCategory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeCategory::pluginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeCategory::categoryIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeCategory::nameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeCategory::visibilityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeCategory::iconChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeCategory::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
