/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeomapitembase_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/quickmapitems/qdeclarativegeomapitembase_p.h"
#include <QtNetwork/QSslError>
#include <QtLocation/private/qdeclarativegeoserviceprovider_p.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeomapitembase_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26QDeclarativeGeoMapItemBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoMapItemBase::qt_create_metaobjectdata<qt_meta_tag_ZN26QDeclarativeGeoMapItemBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoMapItemBase",
        "QML.Element",
        "GeoMapItemBase",
        "QML.AddedInVersion",
        "1280",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "GeoMapItemBase is not intended instantiable by developer.",
        "mapItemOpacityChanged",
        "",
        "addTransitionFinished",
        "removeTransitionFinished",
        "referenceSurfaceChanged",
        "lodThresholdChanged",
        "afterChildrenChanged",
        "afterViewportChanged",
        "QGeoMapViewportChangeEvent",
        "event",
        "polishAndUpdate",
        "baseCameraDataChanged",
        "QGeoCameraData",
        "camera",
        "visibleAreaChanged",
        "geoShape",
        "QGeoShape",
        "autoFadeIn",
        "referenceSurface",
        "QLocation::ReferenceSurface",
        "lodThreshold"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mapItemOpacityChanged'
        QtMocHelpers::SignalData<void()>(9, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addTransitionFinished'
        QtMocHelpers::RevisionedSignalData<void()>(11, 10, QMC::AccessPublic, 0x50c, QMetaType::Void),
        // Signal 'removeTransitionFinished'
        QtMocHelpers::RevisionedSignalData<void()>(12, 10, QMC::AccessPublic, 0x50c, QMetaType::Void),
        // Signal 'referenceSurfaceChanged'
        QtMocHelpers::SignalData<void()>(13, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lodThresholdChanged'
        QtMocHelpers::SignalData<void()>(14, 10, QMC::AccessPublic, QMetaType::Void),
        // Slot 'afterChildrenChanged'
        QtMocHelpers::SlotData<void()>(15, 10, QMC::AccessProtected, QMetaType::Void),
        // Slot 'afterViewportChanged'
        QtMocHelpers::SlotData<void(const QGeoMapViewportChangeEvent &)>(16, 10, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'polishAndUpdate'
        QtMocHelpers::SlotData<void()>(19, 10, QMC::AccessProtected, QMetaType::Void),
        // Slot 'baseCameraDataChanged'
        QtMocHelpers::SlotData<void(const QGeoCameraData &)>(20, 10, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'visibleAreaChanged'
        QtMocHelpers::SlotData<void()>(23, 10, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'geoShape'
        QtMocHelpers::PropertyData<QGeoShape>(24, 0x80000000 | 25, QMC::Readable | QMC::Writable | QMC::Designable | QMC::Scriptable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'autoFadeIn'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, -1, 0x50e),
        // property 'referenceSurface'
        QtMocHelpers::PropertyData<QLocation::ReferenceSurface>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3, 0x606),
        // property 'lodThreshold'
        QtMocHelpers::PropertyData<int>(29, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4, 0x50f),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoMapItemBase, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN26QDeclarativeGeoMapItemBaseE[] = {
    QMetaObject::SuperData::link<QLocation::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QDeclarativeGeoMapItemBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemBaseE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN26QDeclarativeGeoMapItemBaseE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemBaseE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoMapItemBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoMapItemBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mapItemOpacityChanged(); break;
        case 1: _t->addTransitionFinished(); break;
        case 2: _t->removeTransitionFinished(); break;
        case 3: _t->referenceSurfaceChanged(); break;
        case 4: _t->lodThresholdChanged(); break;
        case 5: _t->afterChildrenChanged(); break;
        case 6: _t->afterViewportChanged((*reinterpret_cast<std::add_pointer_t<QGeoMapViewportChangeEvent>>(_a[1]))); break;
        case 7: _t->polishAndUpdate(); break;
        case 8: _t->baseCameraDataChanged((*reinterpret_cast<std::add_pointer_t<QGeoCameraData>>(_a[1]))); break;
        case 9: _t->visibleAreaChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCameraData >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemBase::*)()>(_a, &QDeclarativeGeoMapItemBase::mapItemOpacityChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemBase::*)()>(_a, &QDeclarativeGeoMapItemBase::addTransitionFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemBase::*)()>(_a, &QDeclarativeGeoMapItemBase::removeTransitionFinished, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemBase::*)()>(_a, &QDeclarativeGeoMapItemBase::referenceSurfaceChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMapItemBase::*)()>(_a, &QDeclarativeGeoMapItemBase::lodThresholdChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoShape >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QGeoShape*>(_v) = _t->geoShape(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->autoFadeIn(); break;
        case 2: *reinterpret_cast<QLocation::ReferenceSurface*>(_v) = _t->referenceSurface(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->lodThreshold(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setGeoShape(*reinterpret_cast<QGeoShape*>(_v)); break;
        case 1: _t->setAutoFadeIn(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setReferenceSurface(*reinterpret_cast<QLocation::ReferenceSurface*>(_v)); break;
        case 3: _t->setLodThreshold(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoMapItemBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoMapItemBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26QDeclarativeGeoMapItemBaseE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QDeclarativeGeoMapItemBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoMapItemBase::mapItemOpacityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeGeoMapItemBase::addTransitionFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeoMapItemBase::removeTransitionFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeoMapItemBase::referenceSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeGeoMapItemBase::lodThresholdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
namespace {
struct qt_meta_tag_ZN29QDeclarativeGeoMapPainterPathE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoMapPainterPath::qt_create_metaobjectdata<qt_meta_tag_ZN29QDeclarativeGeoMapPainterPathE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoMapPainterPath"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QDeclarativeGeoMapPainterPath, qt_meta_tag_ZN29QDeclarativeGeoMapPainterPathE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeoMapPainterPath::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickCurve::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativeGeoMapPainterPathE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativeGeoMapPainterPathE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29QDeclarativeGeoMapPainterPathE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoMapPainterPath::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoMapPainterPath *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QDeclarativeGeoMapPainterPath::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoMapPainterPath::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29QDeclarativeGeoMapPainterPathE_t>.strings))
        return static_cast<void*>(this);
    return QQuickCurve::qt_metacast(_clname);
}

int QDeclarativeGeoMapPainterPath::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickCurve::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
