/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeomap_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/quickmapitems/qdeclarativegeomap_p.h"
#include <QtNetwork/QSslError>
#include <QtLocation/private/qdeclarativegeoserviceprovider_p.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeomap_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QDeclarativeGeoMapE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeoMap::qt_create_metaobjectdata<qt_meta_tag_ZN18QDeclarativeGeoMapE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeoMap",
        "QML.Element",
        "Map",
        "QML.AddedInVersion",
        "1280",
        "pluginChanged",
        "",
        "QDeclarativeGeoServiceProvider*",
        "plugin",
        "zoomLevelChanged",
        "zoomLevel",
        "centerChanged",
        "QGeoCoordinate",
        "coordinate",
        "activeMapTypeChanged",
        "supportedMapTypesChanged",
        "minimumZoomLevelChanged",
        "minimumZoomLevel",
        "maximumZoomLevelChanged",
        "maximumZoomLevel",
        "mapItemsChanged",
        "errorChanged",
        "copyrightLinkActivated",
        "link",
        "copyrightsVisibleChanged",
        "visible",
        "colorChanged",
        "QColor",
        "color",
        "bearingChanged",
        "bearing",
        "tiltChanged",
        "tilt",
        "fieldOfViewChanged",
        "fieldOfView",
        "minimumTiltChanged",
        "minimumTilt",
        "maximumTiltChanged",
        "maximumTilt",
        "minimumFieldOfViewChanged",
        "minimumFieldOfView",
        "maximumFieldOfViewChanged",
        "maximumFieldOfView",
        "copyrightsImageChanged",
        "QImage",
        "copyrightsImage",
        "copyrightsChanged",
        "copyrightsHtml",
        "mapReadyChanged",
        "ready",
        "visibleAreaChanged",
        "visibleRegionChanged",
        "mappingManagerInitialized",
        "pluginReady",
        "onSupportedMapTypesChanged",
        "onCameraCapabilitiesChanged",
        "QGeoCameraCapabilities",
        "oldCameraCapabilities",
        "onAttachedCopyrightNoticeVisibilityChanged",
        "onCameraDataChanged",
        "QGeoCameraData",
        "cameraData",
        "setBearing",
        "alignCoordinateToPoint",
        "QPointF",
        "point",
        "removeMapItem",
        "QDeclarativeGeoMapItemBase*",
        "item",
        "addMapItem",
        "addMapItemGroup",
        "QDeclarativeGeoMapItemGroup*",
        "itemGroup",
        "removeMapItemGroup",
        "removeMapItemView",
        "QDeclarativeGeoMapItemView*",
        "itemView",
        "addMapItemView",
        "clearMapItems",
        "toCoordinate",
        "position",
        "clipToViewPort",
        "fromCoordinate",
        "fitViewportToMapItems",
        "QVariantList",
        "items",
        "fitViewportToVisibleMapItems",
        "pan",
        "dx",
        "dy",
        "prefetchData",
        "clearData",
        "fitViewportToGeoShape",
        "QGeoShape",
        "shape",
        "QVariant",
        "margins",
        "activeMapType",
        "QGeoMapType",
        "supportedMapTypes",
        "QList<QGeoMapType>",
        "center",
        "mapItems",
        "QList<QObject*>",
        "error",
        "QGeoServiceProvider::Error",
        "errorString",
        "visibleRegion",
        "copyrightsVisible",
        "mapReady",
        "visibleArea",
        "QRectF"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pluginChanged'
        QtMocHelpers::SignalData<void(QDeclarativeGeoServiceProvider *)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'zoomLevelChanged'
        QtMocHelpers::SignalData<void(qreal)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 10 },
        }}),
        // Signal 'centerChanged'
        QtMocHelpers::SignalData<void(const QGeoCoordinate &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'activeMapTypeChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'supportedMapTypesChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumZoomLevelChanged'
        QtMocHelpers::SignalData<void(qreal)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 17 },
        }}),
        // Signal 'maximumZoomLevelChanged'
        QtMocHelpers::SignalData<void(qreal)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 19 },
        }}),
        // Signal 'mapItemsChanged'
        QtMocHelpers::SignalData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorChanged'
        QtMocHelpers::SignalData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'copyrightLinkActivated'
        QtMocHelpers::SignalData<void(const QString &)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Signal 'copyrightsVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Signal 'colorChanged'
        QtMocHelpers::SignalData<void(const QColor &)>(26, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'bearingChanged'
        QtMocHelpers::SignalData<void(qreal)>(29, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 30 },
        }}),
        // Signal 'tiltChanged'
        QtMocHelpers::SignalData<void(qreal)>(31, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 32 },
        }}),
        // Signal 'fieldOfViewChanged'
        QtMocHelpers::SignalData<void(qreal)>(33, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 34 },
        }}),
        // Signal 'minimumTiltChanged'
        QtMocHelpers::SignalData<void(qreal)>(35, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 36 },
        }}),
        // Signal 'maximumTiltChanged'
        QtMocHelpers::SignalData<void(qreal)>(37, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 38 },
        }}),
        // Signal 'minimumFieldOfViewChanged'
        QtMocHelpers::SignalData<void(qreal)>(39, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 40 },
        }}),
        // Signal 'maximumFieldOfViewChanged'
        QtMocHelpers::SignalData<void(qreal)>(41, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 42 },
        }}),
        // Signal 'copyrightsImageChanged'
        QtMocHelpers::SignalData<void(const QImage &)>(43, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Signal 'copyrightsChanged'
        QtMocHelpers::SignalData<void(const QString &)>(46, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 47 },
        }}),
        // Signal 'mapReadyChanged'
        QtMocHelpers::SignalData<void(bool)>(48, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 49 },
        }}),
        // Signal 'visibleAreaChanged'
        QtMocHelpers::SignalData<void()>(50, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibleRegionChanged'
        QtMocHelpers::RevisionedSignalData<void()>(51, 6, QMC::AccessPublic, 0x50e, QMetaType::Void),
        // Slot 'mappingManagerInitialized'
        QtMocHelpers::SlotData<void()>(52, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pluginReady'
        QtMocHelpers::SlotData<void()>(53, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSupportedMapTypesChanged'
        QtMocHelpers::SlotData<void()>(54, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onCameraCapabilitiesChanged'
        QtMocHelpers::SlotData<void(const QGeoCameraCapabilities &)>(55, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 56, 57 },
        }}),
        // Slot 'onAttachedCopyrightNoticeVisibilityChanged'
        QtMocHelpers::SlotData<void()>(58, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onCameraDataChanged'
        QtMocHelpers::SlotData<void(const QGeoCameraData &)>(59, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 60, 61 },
        }}),
        // Method 'setBearing'
        QtMocHelpers::MethodData<void(qreal, const QGeoCoordinate &)>(62, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 30 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'alignCoordinateToPoint'
        QtMocHelpers::MethodData<void(const QGeoCoordinate &, const QPointF &)>(63, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 64, 65 },
        }}),
        // Method 'removeMapItem'
        QtMocHelpers::MethodData<void(QDeclarativeGeoMapItemBase *)>(66, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 67, 68 },
        }}),
        // Method 'addMapItem'
        QtMocHelpers::MethodData<void(QDeclarativeGeoMapItemBase *)>(69, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 67, 68 },
        }}),
        // Method 'addMapItemGroup'
        QtMocHelpers::MethodData<void(QDeclarativeGeoMapItemGroup *)>(70, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 71, 72 },
        }}),
        // Method 'removeMapItemGroup'
        QtMocHelpers::MethodData<void(QDeclarativeGeoMapItemGroup *)>(73, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 71, 72 },
        }}),
        // Method 'removeMapItemView'
        QtMocHelpers::MethodData<void(QDeclarativeGeoMapItemView *)>(74, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Method 'addMapItemView'
        QtMocHelpers::MethodData<void(QDeclarativeGeoMapItemView *)>(77, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Method 'clearMapItems'
        QtMocHelpers::MethodData<void()>(78, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'toCoordinate'
        QtMocHelpers::MethodData<QGeoCoordinate(const QPointF &, bool) const>(79, 6, QMC::AccessPublic, 0x80000000 | 12, {{
            { 0x80000000 | 64, 80 }, { QMetaType::Bool, 81 },
        }}),
        // Method 'toCoordinate'
        QtMocHelpers::MethodData<QGeoCoordinate(const QPointF &) const>(79, 6, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 12, {{
            { 0x80000000 | 64, 80 },
        }}),
        // Method 'fromCoordinate'
        QtMocHelpers::MethodData<QPointF(const QGeoCoordinate &, bool) const>(82, 6, QMC::AccessPublic, 0x80000000 | 64, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Bool, 81 },
        }}),
        // Method 'fromCoordinate'
        QtMocHelpers::MethodData<QPointF(const QGeoCoordinate &) const>(82, 6, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 64, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Method 'fitViewportToMapItems'
        QtMocHelpers::MethodData<void(const QVariantList &)>(83, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 84, 85 },
        }}),
        // Method 'fitViewportToMapItems'
        QtMocHelpers::MethodData<void()>(83, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'fitViewportToVisibleMapItems'
        QtMocHelpers::MethodData<void()>(86, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'pan'
        QtMocHelpers::MethodData<void(int, int)>(87, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 88 }, { QMetaType::Int, 89 },
        }}),
        // Method 'prefetchData'
        QtMocHelpers::MethodData<void()>(90, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'clearData'
        QtMocHelpers::MethodData<void()>(91, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'fitViewportToGeoShape'
        QtMocHelpers::RevisionedMethodData<void(const QGeoShape &, QVariant)>(92, 6, QMC::AccessPublic, 0x50d, QMetaType::Void, {{
            { 0x80000000 | 93, 94 }, { 0x80000000 | 95, 96 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'plugin'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider*>(8, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'minimumZoomLevel'
        QtMocHelpers::PropertyData<qreal>(17, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'maximumZoomLevel'
        QtMocHelpers::PropertyData<qreal>(19, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'zoomLevel'
        QtMocHelpers::PropertyData<qreal>(10, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'tilt'
        QtMocHelpers::PropertyData<qreal>(32, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'minimumTilt'
        QtMocHelpers::PropertyData<qreal>(36, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'maximumTilt'
        QtMocHelpers::PropertyData<qreal>(38, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'bearing'
        QtMocHelpers::PropertyData<qreal>(30, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'fieldOfView'
        QtMocHelpers::PropertyData<qreal>(34, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'minimumFieldOfView'
        QtMocHelpers::PropertyData<qreal>(40, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'maximumFieldOfView'
        QtMocHelpers::PropertyData<qreal>(42, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'activeMapType'
        QtMocHelpers::PropertyData<QGeoMapType>(97, 0x80000000 | 98, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'supportedMapTypes'
        QtMocHelpers::PropertyData<QList<QGeoMapType>>(99, 0x80000000 | 100, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'center'
        QtMocHelpers::PropertyData<QGeoCoordinate>(101, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'mapItems'
        QtMocHelpers::PropertyData<QList<QObject*>>(102, 0x80000000 | 103, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 7),
        // property 'error'
        QtMocHelpers::PropertyData<QGeoServiceProvider::Error>(104, 0x80000000 | 105, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 8),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(106, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
        // property 'visibleRegion'
        QtMocHelpers::PropertyData<QGeoShape>(107, 0x80000000 | 93, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 23),
        // property 'copyrightsVisible'
        QtMocHelpers::PropertyData<bool>(108, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'color'
        QtMocHelpers::PropertyData<QColor>(28, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'mapReady'
        QtMocHelpers::PropertyData<bool>(109, QMetaType::Bool, QMC::DefaultPropertyFlags, 21),
        // property 'visibleArea'
        QtMocHelpers::PropertyData<QRectF>(110, 0x80000000 | 111, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 22, 0x50c),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeoMap, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN18QDeclarativeGeoMapE[] = {
    QMetaObject::SuperData::link<QGeoServiceProvider::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject QDeclarativeGeoMap::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QDeclarativeGeoMapE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QDeclarativeGeoMapE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN18QDeclarativeGeoMapE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QDeclarativeGeoMapE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeoMap::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeoMap *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pluginChanged((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoServiceProvider*>>(_a[1]))); break;
        case 1: _t->zoomLevelChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 2: _t->centerChanged((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1]))); break;
        case 3: _t->activeMapTypeChanged(); break;
        case 4: _t->supportedMapTypesChanged(); break;
        case 5: _t->minimumZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 6: _t->maximumZoomLevelChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->mapItemsChanged(); break;
        case 8: _t->errorChanged(); break;
        case 9: _t->copyrightLinkActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->copyrightsVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->colorChanged((*reinterpret_cast<std::add_pointer_t<QColor>>(_a[1]))); break;
        case 12: _t->bearingChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 13: _t->tiltChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 14: _t->fieldOfViewChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 15: _t->minimumTiltChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 16: _t->maximumTiltChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 17: _t->minimumFieldOfViewChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 18: _t->maximumFieldOfViewChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 19: _t->copyrightsImageChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 20: _t->copyrightsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->mapReadyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->visibleAreaChanged(); break;
        case 23: _t->visibleRegionChanged(); break;
        case 24: _t->mappingManagerInitialized(); break;
        case 25: _t->pluginReady(); break;
        case 26: _t->onSupportedMapTypesChanged(); break;
        case 27: _t->onCameraCapabilitiesChanged((*reinterpret_cast<std::add_pointer_t<QGeoCameraCapabilities>>(_a[1]))); break;
        case 28: _t->onAttachedCopyrightNoticeVisibilityChanged(); break;
        case 29: _t->onCameraDataChanged((*reinterpret_cast<std::add_pointer_t<QGeoCameraData>>(_a[1]))); break;
        case 30: _t->setBearing((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[2]))); break;
        case 31: _t->alignCoordinateToPoint((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2]))); break;
        case 32: _t->removeMapItem((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoMapItemBase*>>(_a[1]))); break;
        case 33: _t->addMapItem((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoMapItemBase*>>(_a[1]))); break;
        case 34: _t->addMapItemGroup((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoMapItemGroup*>>(_a[1]))); break;
        case 35: _t->removeMapItemGroup((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoMapItemGroup*>>(_a[1]))); break;
        case 36: _t->removeMapItemView((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoMapItemView*>>(_a[1]))); break;
        case 37: _t->addMapItemView((*reinterpret_cast<std::add_pointer_t<QDeclarativeGeoMapItemView*>>(_a[1]))); break;
        case 38: _t->clearMapItems(); break;
        case 39: { QGeoCoordinate _r = _t->toCoordinate((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 40: { QGeoCoordinate _r = _t->toCoordinate((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QGeoCoordinate*>(_a[0]) = std::move(_r); }  break;
        case 41: { QPointF _r = _t->fromCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        case 42: { QPointF _r = _t->fromCoordinate((*reinterpret_cast<std::add_pointer_t<QGeoCoordinate>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->fitViewportToMapItems((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 44: _t->fitViewportToMapItems(); break;
        case 45: _t->fitViewportToVisibleMapItems(); break;
        case 46: _t->pan((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 47: _t->prefetchData(); break;
        case 48: _t->clearData(); break;
        case 49: _t->fitViewportToGeoShape((*reinterpret_cast<std::add_pointer_t<QGeoShape>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCameraData >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDeclarativeGeoMapItemGroup* >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDeclarativeGeoMapItemGroup* >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDeclarativeGeoMapItemView* >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDeclarativeGeoMapItemView* >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCoordinate >(); break;
            }
            break;
        case 49:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoShape >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(QDeclarativeGeoServiceProvider * )>(_a, &QDeclarativeGeoMap::pluginChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::zoomLevelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(const QGeoCoordinate & )>(_a, &QDeclarativeGeoMap::centerChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)()>(_a, &QDeclarativeGeoMap::activeMapTypeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)()>(_a, &QDeclarativeGeoMap::supportedMapTypesChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::minimumZoomLevelChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::maximumZoomLevelChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)()>(_a, &QDeclarativeGeoMap::mapItemsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)()>(_a, &QDeclarativeGeoMap::errorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(const QString & )>(_a, &QDeclarativeGeoMap::copyrightLinkActivated, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(bool )>(_a, &QDeclarativeGeoMap::copyrightsVisibleChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(const QColor & )>(_a, &QDeclarativeGeoMap::colorChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::bearingChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::tiltChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::fieldOfViewChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::minimumTiltChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::maximumTiltChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::minimumFieldOfViewChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(qreal )>(_a, &QDeclarativeGeoMap::maximumFieldOfViewChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(const QImage & )>(_a, &QDeclarativeGeoMap::copyrightsImageChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(const QString & )>(_a, &QDeclarativeGeoMap::copyrightsChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)(bool )>(_a, &QDeclarativeGeoMap::mapReadyChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)()>(_a, &QDeclarativeGeoMap::visibleAreaChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeoMap::*)()>(_a, &QDeclarativeGeoMap::visibleRegionChanged, 23))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoCoordinate >(); break;
        case 17:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QGeoShape >(); break;
        case 14:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QObject*> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v) = _t->plugin(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->minimumZoomLevel(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->maximumZoomLevel(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->zoomLevel(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->tilt(); break;
        case 5: *reinterpret_cast<qreal*>(_v) = _t->minimumTilt(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->maximumTilt(); break;
        case 7: *reinterpret_cast<qreal*>(_v) = _t->bearing(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->fieldOfView(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->minimumFieldOfView(); break;
        case 10: *reinterpret_cast<qreal*>(_v) = _t->maximumFieldOfView(); break;
        case 11: *reinterpret_cast<QGeoMapType*>(_v) = _t->activeMapType(); break;
        case 12: *reinterpret_cast<QList<QGeoMapType>*>(_v) = _t->supportedMapTypes(); break;
        case 13: *reinterpret_cast<QGeoCoordinate*>(_v) = _t->center(); break;
        case 14: *reinterpret_cast<QList<QObject*>*>(_v) = _t->mapItems(); break;
        case 15: *reinterpret_cast<QGeoServiceProvider::Error*>(_v) = _t->error(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 17: *reinterpret_cast<QGeoShape*>(_v) = _t->visibleRegion(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->copyrightsVisible(); break;
        case 19: *reinterpret_cast<QColor*>(_v) = _t->color(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->mapReady(); break;
        case 21: *reinterpret_cast<QRectF*>(_v) = _t->visibleArea(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlugin(*reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v)); break;
        case 1: _t->setMinimumZoomLevel(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setMaximumZoomLevel(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setZoomLevel(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setTilt(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setMinimumTilt(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setMaximumTilt(*reinterpret_cast<qreal*>(_v)); break;
        case 7: _t->setBearing(*reinterpret_cast<qreal*>(_v)); break;
        case 8: _t->setFieldOfView(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setMinimumFieldOfView(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setMaximumFieldOfView(*reinterpret_cast<qreal*>(_v)); break;
        case 11: _t->setActiveMapType(*reinterpret_cast<QGeoMapType*>(_v)); break;
        case 13: _t->setCenter(*reinterpret_cast<QGeoCoordinate*>(_v)); break;
        case 17: _t->setVisibleRegion(*reinterpret_cast<QGeoShape*>(_v)); break;
        case 18: _t->setCopyrightsVisible(*reinterpret_cast<bool*>(_v)); break;
        case 19: _t->setColor(*reinterpret_cast<QColor*>(_v)); break;
        case 21: _t->setVisibleArea(*reinterpret_cast<QRectF*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeoMap::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeoMap::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QDeclarativeGeoMapE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int QDeclarativeGeoMap::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 50)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 50;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeoMap::pluginChanged(QDeclarativeGeoServiceProvider * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QDeclarativeGeoMap::zoomLevelChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QDeclarativeGeoMap::centerChanged(const QGeoCoordinate & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QDeclarativeGeoMap::activeMapTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeGeoMap::supportedMapTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeGeoMap::minimumZoomLevelChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QDeclarativeGeoMap::maximumZoomLevelChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QDeclarativeGeoMap::mapItemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativeGeoMap::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QDeclarativeGeoMap::copyrightLinkActivated(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QDeclarativeGeoMap::copyrightsVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QDeclarativeGeoMap::colorChanged(const QColor & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QDeclarativeGeoMap::bearingChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QDeclarativeGeoMap::tiltChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QDeclarativeGeoMap::fieldOfViewChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QDeclarativeGeoMap::minimumTiltChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QDeclarativeGeoMap::maximumTiltChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QDeclarativeGeoMap::minimumFieldOfViewChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void QDeclarativeGeoMap::maximumFieldOfViewChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void QDeclarativeGeoMap::copyrightsImageChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void QDeclarativeGeoMap::copyrightsChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void QDeclarativeGeoMap::mapReadyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void QDeclarativeGeoMap::visibleAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QDeclarativeGeoMap::visibleRegionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}
QT_WARNING_POP
