/****************************************************************************
** Meta object code from reading C++ file 'qgeomap_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/maps/qgeomap_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qgeomap_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7QGeoMapE_t {};
} // unnamed namespace

template <> constexpr inline auto QGeoMap::qt_create_metaobjectdata<qt_meta_tag_ZN7QGeoMapE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QGeoMap",
        "cameraDataChanged",
        "",
        "QGeoCameraData",
        "cameraData",
        "sgNodeChanged",
        "activeMapTypeChanged",
        "cameraCapabilitiesChanged",
        "QGeoCameraCapabilities",
        "oldCameraCapabilities",
        "copyrightsImageChanged",
        "QImage",
        "copyrightsImage",
        "copyrightsChanged",
        "copyrightsHtml",
        "copyrightsStyleSheetChanged",
        "styleSheet",
        "visibleAreaChanged",
        "Capability",
        "SupportsNothing",
        "SupportsVisibleRegion",
        "SupportsSetBearing",
        "SupportsAnchoringCoordinate",
        "SupportsFittingViewportToGeoRectangle",
        "SupportsVisibleArea",
        "Capabilities"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'cameraDataChanged'
        QtMocHelpers::SignalData<void(const QGeoCameraData &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'sgNodeChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeMapTypeChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cameraCapabilitiesChanged'
        QtMocHelpers::SignalData<void(const QGeoCameraCapabilities &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'copyrightsImageChanged'
        QtMocHelpers::SignalData<void(const QImage &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'copyrightsChanged'
        QtMocHelpers::SignalData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Signal 'copyrightsStyleSheetChanged'
        QtMocHelpers::SignalData<void(const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'visibleAreaChanged'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Capability'
        QtMocHelpers::EnumData<enum Capability>(18, 18, QMC::EnumFlags{}).add({
            {   19, Capability::SupportsNothing },
            {   20, Capability::SupportsVisibleRegion },
            {   21, Capability::SupportsSetBearing },
            {   22, Capability::SupportsAnchoringCoordinate },
            {   23, Capability::SupportsFittingViewportToGeoRectangle },
            {   24, Capability::SupportsVisibleArea },
        }),
        // flag 'Capabilities'
        QtMocHelpers::EnumData<Capabilities>(25, 18, QMC::EnumIsFlag).add({
            {   19, Capability::SupportsNothing },
            {   20, Capability::SupportsVisibleRegion },
            {   21, Capability::SupportsSetBearing },
            {   22, Capability::SupportsAnchoringCoordinate },
            {   23, Capability::SupportsFittingViewportToGeoRectangle },
            {   24, Capability::SupportsVisibleArea },
        }),
    };
    return QtMocHelpers::metaObjectData<QGeoMap, qt_meta_tag_ZN7QGeoMapE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QGeoMap::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QGeoMapE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QGeoMapE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7QGeoMapE_t>.metaTypes,
    nullptr
} };

void QGeoMap::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QGeoMap *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cameraDataChanged((*reinterpret_cast<std::add_pointer_t<QGeoCameraData>>(_a[1]))); break;
        case 1: _t->sgNodeChanged(); break;
        case 2: _t->activeMapTypeChanged(); break;
        case 3: _t->cameraCapabilitiesChanged((*reinterpret_cast<std::add_pointer_t<QGeoCameraCapabilities>>(_a[1]))); break;
        case 4: _t->copyrightsImageChanged((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1]))); break;
        case 5: _t->copyrightsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->copyrightsStyleSheetChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->visibleAreaChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)(const QGeoCameraData & )>(_a, &QGeoMap::cameraDataChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)()>(_a, &QGeoMap::sgNodeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)()>(_a, &QGeoMap::activeMapTypeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)(const QGeoCameraCapabilities & )>(_a, &QGeoMap::cameraCapabilitiesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)(const QImage & )>(_a, &QGeoMap::copyrightsImageChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)(const QString & )>(_a, &QGeoMap::copyrightsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)(const QString & )>(_a, &QGeoMap::copyrightsStyleSheetChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QGeoMap::*)()>(_a, &QGeoMap::visibleAreaChanged, 7))
            return;
    }
}

const QMetaObject *QGeoMap::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QGeoMap::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QGeoMapE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QGeoMap::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void QGeoMap::cameraDataChanged(const QGeoCameraData & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QGeoMap::sgNodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QGeoMap::activeMapTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QGeoMap::cameraCapabilitiesChanged(const QGeoCameraCapabilities & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QGeoMap::copyrightsImageChanged(const QImage & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QGeoMap::copyrightsChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QGeoMap::copyrightsStyleSheetChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QGeoMap::visibleAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
