/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativegeocodemodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtlocation/src/location/declarativemaps/qdeclarativegeocodemodel_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativegeocodemodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24QDeclarativeGeocodeModelE_t {};
} // unnamed namespace

template <> constexpr inline auto QDeclarativeGeocodeModel::qt_create_metaobjectdata<qt_meta_tag_ZN24QDeclarativeGeocodeModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QDeclarativeGeocodeModel",
        "QML.Element",
        "GeocodeModel",
        "countChanged",
        "",
        "pluginChanged",
        "statusChanged",
        "errorChanged",
        "locationsChanged",
        "autoUpdateChanged",
        "boundsChanged",
        "queryChanged",
        "limitChanged",
        "offsetChanged",
        "update",
        "queryContentChanged",
        "geocodeFinished",
        "QGeoCodeReply*",
        "reply",
        "geocodeError",
        "QGeoCodeReply::Error",
        "error",
        "errorString",
        "pluginReady",
        "get",
        "QDeclarativeGeoLocation*",
        "index",
        "reset",
        "cancel",
        "plugin",
        "QDeclarativeGeoServiceProvider*",
        "autoUpdate",
        "status",
        "Status",
        "count",
        "limit",
        "offset",
        "query",
        "QVariant",
        "bounds",
        "GeocodeError",
        "Null",
        "Ready",
        "Loading",
        "Error",
        "NoError",
        "EngineNotSetError",
        "CommunicationError",
        "ParseError",
        "UnsupportedOptionError",
        "CombinationError",
        "UnknownError",
        "UnknownParameterError",
        "MissingRequiredParameterError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pluginChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'errorChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'locationsChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoUpdateChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'boundsChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'queryChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'limitChanged'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'offsetChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'queryContentChanged'
        QtMocHelpers::SlotData<void()>(15, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'geocodeFinished'
        QtMocHelpers::SlotData<void(QGeoCodeReply *)>(16, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'geocodeError'
        QtMocHelpers::SlotData<void(QGeoCodeReply *, QGeoCodeReply::Error, const QString &)>(19, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 20, 21 }, { QMetaType::QString, 22 },
        }}),
        // Slot 'pluginReady'
        QtMocHelpers::SlotData<void()>(23, 4, QMC::AccessProtected, QMetaType::Void),
        // Method 'get'
        QtMocHelpers::MethodData<QDeclarativeGeoLocation *(int)>(24, 4, QMC::AccessPublic, 0x80000000 | 25, {{
            { QMetaType::Int, 26 },
        }}),
        // Method 'reset'
        QtMocHelpers::MethodData<void()>(27, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'cancel'
        QtMocHelpers::MethodData<void()>(28, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'plugin'
        QtMocHelpers::PropertyData<QDeclarativeGeoServiceProvider*>(29, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'autoUpdate'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(32, 0x80000000 | 33, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'errorString'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'count'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'limit'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'offset'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'query'
        QtMocHelpers::PropertyData<QVariant>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'bounds'
        QtMocHelpers::PropertyData<QVariant>(39, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'error'
        QtMocHelpers::PropertyData<enum GeocodeError>(21, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(33, 33, QMC::EnumFlags{}).add({
            {   41, Status::Null },
            {   42, Status::Ready },
            {   43, Status::Loading },
            {   44, Status::Error },
        }),
        // enum 'GeocodeError'
        QtMocHelpers::EnumData<enum GeocodeError>(40, 40, QMC::EnumFlags{}).add({
            {   45, GeocodeError::NoError },
            {   46, GeocodeError::EngineNotSetError },
            {   47, GeocodeError::CommunicationError },
            {   48, GeocodeError::ParseError },
            {   49, GeocodeError::UnsupportedOptionError },
            {   50, GeocodeError::CombinationError },
            {   51, GeocodeError::UnknownError },
            {   52, GeocodeError::UnknownParameterError },
            {   53, GeocodeError::MissingRequiredParameterError },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QDeclarativeGeocodeModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QDeclarativeGeocodeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QDeclarativeGeocodeModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QDeclarativeGeocodeModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QDeclarativeGeocodeModelE_t>.metaTypes,
    nullptr
} };

void QDeclarativeGeocodeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QDeclarativeGeocodeModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->pluginChanged(); break;
        case 2: _t->statusChanged(); break;
        case 3: _t->errorChanged(); break;
        case 4: _t->locationsChanged(); break;
        case 5: _t->autoUpdateChanged(); break;
        case 6: _t->boundsChanged(); break;
        case 7: _t->queryChanged(); break;
        case 8: _t->limitChanged(); break;
        case 9: _t->offsetChanged(); break;
        case 10: _t->update(); break;
        case 11: _t->queryContentChanged(); break;
        case 12: _t->geocodeFinished((*reinterpret_cast<std::add_pointer_t<QGeoCodeReply*>>(_a[1]))); break;
        case 13: _t->geocodeError((*reinterpret_cast<std::add_pointer_t<QGeoCodeReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QGeoCodeReply::Error>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 14: _t->pluginReady(); break;
        case 15: { QDeclarativeGeoLocation* _r = _t->get((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDeclarativeGeoLocation**>(_a[0]) = std::move(_r); }  break;
        case 16: _t->reset(); break;
        case 17: _t->cancel(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCodeReply* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QGeoCodeReply* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::countChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::pluginChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::statusChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::errorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::locationsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::autoUpdateChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::boundsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::queryChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::limitChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QDeclarativeGeocodeModel::*)()>(_a, &QDeclarativeGeocodeModel::offsetChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeGeoServiceProvider* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v) = _t->plugin(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->autoUpdate(); break;
        case 2: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->errorString(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->limit(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->offset(); break;
        case 7: *reinterpret_cast<QVariant*>(_v) = _t->query(); break;
        case 8: *reinterpret_cast<QVariant*>(_v) = _t->bounds(); break;
        case 9: *reinterpret_cast<enum GeocodeError*>(_v) = _t->error(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPlugin(*reinterpret_cast<QDeclarativeGeoServiceProvider**>(_v)); break;
        case 1: _t->setAutoUpdate(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setLimit(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setOffset(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setQuery(*reinterpret_cast<QVariant*>(_v)); break;
        case 8: _t->setBounds(*reinterpret_cast<QVariant*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QDeclarativeGeocodeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeGeocodeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QDeclarativeGeocodeModelE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QDeclarativeGeocodeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QDeclarativeGeocodeModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeGeocodeModel::pluginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeGeocodeModel::statusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeGeocodeModel::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeGeocodeModel::locationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeGeocodeModel::autoUpdateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeGeocodeModel::boundsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativeGeocodeModel::queryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QDeclarativeGeocodeModel::limitChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QDeclarativeGeocodeModel::offsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}
QT_WARNING_POP
